"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressHttpAdapter = void 0;
const express_1 = __importDefault(require("express"));
require("express-async-errors");
const Http_1 = require("../entities/Http");
const HttpContext_1 = require("../entities/HttpContext");
const HttpResponse_1 = require("../entities/HttpResponse");
const multer_1 = __importDefault(require("multer"));
const cors_1 = __importDefault(require("cors"));
const crypto_1 = require("crypto");
class ExpressHttpAdapter extends Http_1.Http {
    constructor(uploadPath, corsOptions = {}) {
        super();
        this.app = (0, express_1.default)();
        this.app.use((0, cors_1.default)(corsOptions));
        this.app.use(express_1.default.json());
        this.app.use(express_1.default.urlencoded({ extended: true }));
        this.upload = (0, multer_1.default)({
            dest: uploadPath,
            storage: multer_1.default.diskStorage({
                destination: uploadPath,
                filename: (req, file, callback) => {
                    const filename = `${(0, crypto_1.randomUUID)()}-${file.originalname}`;
                    return callback(null, filename);
                },
            }),
        });
    }
    sendResponse(res, response) {
        for (const key in response.headers) {
            res.setHeader(key, response.headers[key]);
        }
        res.status(response.status).send(response.body);
    }
    addGlobalMiddleware(middleware) {
        this.app.use(async (req, res, next) => {
            const data = await middleware(new HttpContext_1.HttpContext(req.url, req.query, req.body, req.params));
            if (data instanceof HttpResponse_1.HttpResponse) {
                return this.sendResponse(res, data);
            }
            next();
        });
    }
    on(path, method, handle, middlewares, options) {
        this.app[method](path, options?.multipartField ? this.upload.single(options.multipartField) : this.upload.none(), async (req, res) => {
            const ctx = new HttpContext_1.HttpContext(req.url, req.query, req.body, req.params, req.headers, req.file?.filename);
            try {
                if (middlewares) {
                    for (const fn of middlewares) {
                        const data = await fn(ctx);
                        if (data instanceof HttpResponse_1.HttpResponse) {
                            return this.sendResponse(res, data);
                        }
                    }
                }
                const response = await handle(ctx);
                return this.sendResponse(res, response);
            }
            catch (error) {
                if (error instanceof Error && this.errorHandler) {
                    return this.sendResponse(res, await this.errorHandler(error, ctx));
                }
                throw error;
            }
        });
    }
    listen(port, callback) {
        this.app.listen(port, callback);
    }
}
exports.ExpressHttpAdapter = ExpressHttpAdapter;
